SystemOrganization addCategory: #'GT-Tests-Debugger'!

TestCase subclass: #GTDebuggerSmokeTest
	instanceVariableNames: 'session window debugger'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Debugger'!

!GTDebuggerSmokeTest commentStamp: 'AndreiChis 1/22/2016 20:06' prior: 0!
I provide support for testing GT Debuggers.  

Each subclass can provide test for a debugger. To indicate the debugger a debugger needs to override #debuggerToTest.

In a test subclasses should first create a debugger using #initializeSessionForContext:withStepBlock:.
To finish a test subclasses should use #assertDebuggerClosing. !

GTDebuggerSmokeTest subclass: #GTBytecodeDebuggerSmokeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Debugger'!

!GTBytecodeDebuggerSmokeTest methodsFor: 'accessing' stamp: 'AndreiChis 12/28/2015 20:41'!
debuggerToTest

	^ GTBytecodeDebugger! !

!GTBytecodeDebuggerSmokeTest methodsFor: 'tests' stamp: 'AndreiChis 1/22/2016 18:51'!
testDebugger

	self 
		assertDebuggerInitializationForContext: [ 20 factorial ] asContext 
		andStepBlock: [:aProcess | "The bytecode debugger can handle any context" ].

 	self assertDebuggerClosing.! !

!GTDebuggerSmokeTest class methodsFor: 'testing' stamp: 'AndreiChis 12/17/2013 10:00'!
isAbstract
	^ self name = #GTDebuggerSmokeTest! !

!GTDebuggerSmokeTest methodsFor: 'asserting' stamp: 'AndreiChis 1/22/2016 18:49'!
assertDebuggerClosing
	
 	debugger close.

	self assert: window model equals: nil.
	self assert: debugger session interruptedContext equals: nil.
	self assert: debugger session interruptedProcess equals: nil.! !

!GTDebuggerSmokeTest methodsFor: 'asserting' stamp: 'AndreiChis 1/22/2016 18:49'!
assertDebuggerInitialization
	
	self assert: (debugger class handlesContext: session interruptedContext).
	
	window := debugger openOn: session.
	
	self assert: window model equals: debugger.
	self assert: debugger session interruptedContext notNil.
	self assert: debugger session interruptedProcess notNil.
	! !

!GTDebuggerSmokeTest methodsFor: 'asserting' stamp: 'AndreiChis 1/22/2016 18:53'!
assertDebuggerInitializationForContext: aContext andStepBlock: aBlock

	self 
		initializeSessionForContext: aContext
		withStepBlock: aBlock.

	self assertDebuggerInitialization.! !

!GTDebuggerSmokeTest methodsFor: 'accessing' stamp: 'AndreiChis 12/16/2013 19:17'!
debuggerToTest
	self subclassResponsability! !

!GTDebuggerSmokeTest methodsFor: 'initialization' stamp: 'AndreiChis 1/22/2016 18:53'!
initializeSessionForContext: aContext withStepBlock: aBlock
	| process |

	process := Process 
		forContext: aContext
		priority: Processor userInterruptPriority.
	
	aBlock value: process.
	session := self debuggerToTest spanNewSessionLabeled:  'test session' forContext: process suspendedContext fromProcess: process! !

!GTDebuggerSmokeTest methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 18:29'!
setUp
	debugger := self debuggerToTest new.! !

!GTDebuggerSmokeTest methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 18:52'!
tearDown 
	debugger ifNotNil: [ debugger close].
	window := nil.
	debugger := nil.
	session := nil.! !

GTDebuggerSmokeTest subclass: #GTGenericStackDebuggerSmokeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Debugger'!

!GTGenericStackDebuggerSmokeTest methodsFor: 'accessing' stamp: 'AndreiChis 12/17/2013 11:24'!
debuggerToTest

	^ GTGenericStackDebugger! !

!GTGenericStackDebuggerSmokeTest methodsFor: 'tests' stamp: 'AndreiChis 1/22/2016 18:51'!
testDebugger

	self 
		assertDebuggerInitializationForContext: [ 20 factorial ] asContext 
		andStepBlock: [:aProcess | "The generic debugger can handle any context" ].

 	self assertDebuggerClosing.! !

GTDebuggerSmokeTest subclass: #GTSUnitDebuggerSmokeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Debugger'!

!GTSUnitDebuggerSmokeTest methodsFor: 'accessing' stamp: 'AndreiChis 12/28/2015 20:56'!
debuggerToTest

	^ GTSUnitDebugger! !

!GTSUnitDebuggerSmokeTest methodsFor: 'utils' stamp: 'AndreiChis 1/22/2016 18:46'!
stepBlockForTestArrays

	^ [ :aProcess |
		aProcess
			"Step into the execution of the test method"
			completeStep: aProcess suspendedContext;
			completeStep: aProcess suspendedContext;	
			step;
			
			"Step into the assert method"
			completeStep: aProcess suspendedContext; 
			completeStep: aProcess suspendedContext;
			completeStep: aProcess suspendedContext;
			step ]
		
! !

!GTSUnitDebuggerSmokeTest methodsFor: 'utils' stamp: 'AndreiChis 1/22/2016 18:45'!
stepBlockForTestWithHalt

	^ [ :aProcess |
		aProcess 
			"Step into the execution of the test method"
			completeStep: aProcess suspendedContext;
			completeStep: aProcess suspendedContext;	
			step;
		
			"Step until before the message send #halt"
			stepToSendOrReturn;
			completeStep: aProcess suspendedContext;
			stepToSendOrReturn ]
		
! !

!GTSUnitDebuggerSmokeTest methodsFor: 'tests' stamp: 'AndreiChis 1/22/2016 18:44'!
testAssertionEqualsDetected

	self 
		assertDebuggerInitializationForContext: [ GTSUnitExampleFailingTest new testArrays ] asContext 
		andStepBlock: self stepBlockForTestArrays.

	self assert: session testContext method equals: GTSUnitExampleFailingTest >> #testArrays.
	self assert: session assertionContext method selector equals: #assert:equals:.
	self assert: session assertionCallerContext equals: session testContext.
	
	self assert: session supportsDiff description: 'When #assert:equals: is encountered a diff can be used'.

 	self assertDebuggerClosing.! !

!GTSUnitDebuggerSmokeTest methodsFor: 'tests' stamp: 'AndreiChis 1/22/2016 18:48'!
testSuccessfullTestWithHalt

	self 
		assertDebuggerInitializationForContext: [ GTSUnitExampleFailingTest new testWithHalt ] asContext 
		andStepBlock: self stepBlockForTestWithHalt.

	self assert: session testContext method equals: GTSUnitExampleFailingTest >> #testWithHalt.
	self assert: session assertionContext equals: nil.
	self assert: session assertionCallerContext equals: nil.
	
	self deny: session supportsDiff description: 'When there is no assertion no diff is supported'.

 	self assertDebuggerClosing.! !

TestCase subclass: #GTGenericStackDebuggerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Debugger'!

!GTGenericStackDebuggerTest methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/8/2016 16:08'!
testBasic
	| context process debugger printedString |
	context := [ 20 factorial ] asContext.
	process := Process
		forContext: context
		priority: Processor userInterruptPriority.
	debugger := GTGenericStackDebugger on: (process newDebugSessionNamed: 'test debugging' startedAt: context).
	self assert: debugger stackPresentation selection equals: context.
	printedString := '[ 20 factorial ] in GTGenericStackDebuggerTest>>testBasic'.
	self assert: debugger selectedContext printString equals: printedString.
	debugger send.
	debugger send.
	debugger update.
	self
		assert: debugger codePresentation entity sourceCode equals: (Integer >> #factorial) sourceCode.
	self
		assert:
			debugger stackPresentation selection printString
		equals: 'SmallInteger(Integer)>>factorial'.
	process terminate! !

GLMMorphicTest subclass: #GTGenericStackDebuggerUITest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Debugger'!

!GTGenericStackDebuggerUITest methodsFor: 'testing' stamp: 'AndreiChis 1/18/2016 13:57'!
testNoOverlappingCodeEditorKeyBindings
	<expectedFailure>
	| context process codeEditor keymaps doubleKeymaps |
	context := [ 20 factorial ] asContext.
	process := Process
		forContext: context
		priority: Processor userInterruptPriority.
	window := GTGenericStackDebugger openOn: (process newDebugSessionNamed: 'test debugging' startedAt: context).
	codeEditor := self findSatisfying: [ :morph | 
		morph model isKindOf: GLMRubricSmalltalkTextModel ] in: window.
	keymaps := OrderedCollection new.
	codeEditor withAllOwnersDo: [:morph | 
		keymaps addAll: morph kmDispatcher directKeymaps allEntries keymaps ].
	doubleKeymaps := (keymaps groupedBy: [:each | each shortcut ]) select: [ :each | each value size > 1 ].
	self assert: doubleKeymaps isEmpty! !
